@@ -15,6 +15,34 @@ from .. import models
 from .. import signals
 
 
+class BackendTestCase(TestCase):
+    def test_get_form_class_raises_NotYetImplemented(self):
+        backend = backends.Backend()
+        self.assertRaises(NotImplementedError, backend.get_form_class)
+
+    def test_purchase_raises_NotYetImplemented(self):
+        backend = backends.Backend()
+        donation, form = self.random_donation_and_form
+        self.assertRaises(NotImplementedError, backend.purchase,
+                donation, form)
+
+    def test_fires_donation_complete_on_successful_charge(self):
+        donation, form = self.random_donation_and_form
+        result = {
+            "status": True,
+            "random": random.randint(100, 200),
+        }
+        backend = backends.Backend()
+        signal = (fudge.Fake().expects_call()
+                .with_args(sender=backend, donation=donation,
+                        signal=arg.any(), form=form, result=result))
+        signals.successful_purchase.connect(signal)
+        backend.send_successful_purchase(donation, form, result)
+
+        fudge.verify()
+        signals.successful_purchase.disconnect(signal)
+
+
 class AuthorizeNetBackendTestCase(TestCase):
     def get_api_stub(self, response=None, reason_text=None, successful=True):
         if response is None:
@@ -205,6 +233,34 @@ class AuthorizeNetBackendTestCase(TestCase):
             backend.purchase(donation, donation_form)
         fudge.verify()
 
+    def test_includes_donation_type_if_provided_for_transaction(self):
+        donation, donation_form = self.random_donation_and_form
+        donation.donation_type = self.random_monthly_type
+
+        api = fudge.Fake("api")
+        api.expects("transaction").with_args(
+                amount=donation.amount,
+                card_num=donation_form.cleaned_data["card_number"],
+                card_code=donation_form.cleaned_data["ccv_code"],
+                exp_date=u"%02d-%04d" % (
+                        int(donation_form.cleaned_data["expiration_month"]),
+                        int(donation_form.cleaned_data["expiration_year"])),
+                description=u"Membership: %s" % donation.donation_type.name,
+                first_name=unicode(donation.donor.first_name),
+                last_name=unicode(donation.donor.last_name),
+                address=donation.donor.address.address,
+                city=donation.donor.address.city,
+                state=donation.donor.address.state,
+                zip=donation.donor.address.zipcode,
+        ).returns({"reason_code": u"1", "reason_text": u"Some random Reason"})
+        get_api = fudge.Fake().expects_call().returns(api)
+
+        backend = backends.AuthorizeNetBackend()
+        with fudge.patched_context(backend, "get_api", get_api):
+            backend.purchase(donation, donation_form)
+        fudge.verify()
+
+
     def test_adds_test_request_to_kwargs_if_in_testing_mode(self):
         donation, donation_form = self.random_donation_and_form
 
