@@ -1,5 +1,7 @@
+from django.http import QueryDict
 import fudge
 import random
+from urllib import urlencode
 from ._utils import TestCase
 
 from .. import constants
@@ -7,7 +9,14 @@ from .. import forms
 from .. import models
 
 
-class BaseDonationFormTestCase(TestCase):
+class DjangoFormAssertionsMixin(object):
+    def assert_field_error(self, form, field_name):
+        self.assertFalse(form.is_valid(), msg="sanity check")
+        self.assert_(field_name in form.errors,
+                msg="%s not in form errors" % field_name)
+
+
+class BaseDonationFormTestCase(DjangoFormAssertionsMixin, TestCase):
     def test_attribution_is_stored(self):
         random_attribution = "Random Attribution %d" % random.randint(100, 200)
         data = self.get_base_random_data()
@@ -217,6 +226,17 @@ class BaseDonationFormTestCase(TestCase):
         form = forms.BaseDonationForm(data=data)
         self.assertTrue(form.is_valid())
 
+    def test_cleaned_data_is_available_with_dt_and_no_amount(self):
+        donation_type = self.random_type
+        address_kwargs = self.random_address_kwargs
+        data = self.get_base_random_data(donation_type_pk=donation_type.pk)
+        data.update(self.prefix_data(address_kwargs), prefix="billing")
+        del data["amount"]
+
+        form = forms.BaseDonationForm(data=data)
+        form.is_valid()
+        self.assertEqual(form.cleaned_data["amount"], donation_type.amount)
+
     def test_is_valid_returns_false_with_invalid_donation_type(self):
         address_kwargs = self.random_address_kwargs
         data = self.get_base_random_data(
@@ -251,19 +271,72 @@ class BaseDonationFormTestCase(TestCase):
         form = MyAwesomeForm(data=data)
         self.assertFalse(form.is_valid())
 
+    def test_has_a_donor_form_property(self):
+        form = forms.BaseDonationForm()
+        self.assertIsA(form.donor_form, forms.DonorForm)
 
+    def test_is_valid_returns_false_if_donor_form_invalid_donor(self):
+        for field_name in ["first_name", "last_name", ]:
+            data = self.get_base_random_data()
+            del data[field_name]
+            form = forms.BaseDonationForm(data=data)
+            self.assertFalse(form.is_valid())
+            self.assert_field_error(form.donor_form, field_name)
 
 
 class CreditCardDonationFormTestCase(TestCase):
+    form_class = forms.CreditCardDonationForm
+
     def setUp(self):
-        data = self.get_base_random_data()
-        self.donor = self.random_donor
-        data = self.get_base_random_data(name=self.donor.name)
+        name_kwargs = self.random_address_kwargs
+        data = self.get_base_random_data(**name_kwargs)
         self.amount = data["amount"]
-        self.donation_form = forms.CreditCardDonationForm(data)
         self.card_number = data["card_number"]
         self.ccv_code = data["ccv_code"]
         self.expiration_month = data["expiration_month"]
         self.expiration_year = data["expiration_year"]
+        self.data = data
 
-    # TODO: test get_data_for_charge directly
+    def get_form(self, *args, **kwargs):
+        return self.form_class(*args, **kwargs)
+
+    def get_invalid_form(self):
+        del self.data["amount"]
+        return self.get_form(data=self.data)
+
+    def assert_value_is_empty(self, field):
+        form = self.get_invalid_form()
+        form.data = QueryDict(urlencode(form.data))
+        form.fields_to_strip = ["card_number", "ccv_code"]
+        self.assertFalse(form.is_valid())
+        self.assertEqual("", form[field].value())
+
+    def assert_value_is_not_empty(self, field):
+        form = self.get_invalid_form()
+        form.fields_to_strip = []
+        self.assertFalse(form.is_valid())
+        self.assertNotEqual("", form[field].value())
+
+    def test_clears_credit_card_numbers_if_form_is_invalid(self):
+        self.assert_value_is_empty("card_number")
+
+    def test_clears_ccv_card_numbers_if_form_is_invalid(self):
+        self.assert_value_is_empty("ccv_code")
+
+    def test_does_not_clear_card_numbers_if_form_is_invalid(self):
+        self.assert_value_is_not_empty("card_number")
+
+    def test_does_not_clear_ccv_codes_if_form_is_invalid(self):
+        self.assert_value_is_not_empty("ccv_code")
+
+    def test_can_configure_the_fields_to_strip(self):
+        form = self.get_invalid_form()
+        form.fields_to_strip = ["card_number", ]
+        form.is_valid()
+
+        self.assertEqual("", form["card_number"].value())
+        self.assertNotEqual("", form["ccv_code"].value())
+
+
+class AuthorizeDonationFormTestCase(CreditCardDonationFormTestCase):
+    form_class = forms.AuthorizeDonationForm
